"""

Cython wrapper for some processing scripts

(c) Benjamin Irving 20150713

"""

from __future__ import division, print_function, absolute_import

from libcpp.vector cimport vector
import numpy as np
cimport numpy as cnp

cdef extern from "processing.h":
    vector[int] get_mean_point_distance(vector[int] &, vector[int] &, vector[int] &)

def get_mpd(cnp.ndarray[int, ndim=1] x, cnp.ndarray[int, ndim=1] y, cnp.ndarray[int, ndim=1] z):
    """

    Get the maximum distance between two points in the region

    :return:
    """

    dist = get_mean_point_distance(x, y, z)
    return np.array(dist)


from __future__ import division, print_function, absolute_import

__author__ = 'Ben Irving'

from .create_im import createlab
from .create_im import grouplab

__all__ = ['createlab', 'grouplab', 'plot_region', 'save_nifti']



"""
Quantiphyse - Widget for data clustering using the Supervoxel method

Copyright (c) 2013-2018 University of Oxford
"""

from PySide import QtGui

from quantiphyse.gui.widgets import QpWidget, TitleWidget, Citation, OverlayCombo, RoiCombo, NumericOption

from .process import SupervoxelsProcess

CITE_TITLE = "maskSLIC: Regional Superpixel Generation with Application to Local Pathology Characterisation in Medical Images"
CITE_AUTHOR = "Benjamin Irving"
CITE_JOURNAL = "https://arxiv.org/abs/1606.09518v2 (2017)"

class PerfSlicWidget(QpWidget):
    """
    Generates supervoxels using SLIC method
    """
    def __init__(self, **kwargs):
        super(PerfSlicWidget, self).__init__(name="Super Voxels", icon="sv", 
                                             desc="Generate supervoxel clusters", 
                                             group="Clustering", **kwargs)
        
    def init_ui(self):
        layout = QtGui.QVBoxLayout()
        self.setLayout(layout)

        title = TitleWidget(self, "Supervoxel Generation", help="sv")
        layout.addWidget(title)
        
        cite = Citation(CITE_TITLE, CITE_AUTHOR, CITE_JOURNAL)
        layout.addWidget(cite)

        hbox = QtGui.QHBoxLayout()
        optbox = QtGui.QGroupBox()
        optbox.setTitle("Options")
        grid = QtGui.QGridLayout()
        optbox.setLayout(grid)
        
        grid.addWidget(QtGui.QLabel("Data"), 0, 0)
        self.ovl = OverlayCombo(self.ivm)
        self.ovl.currentIndexChanged.connect(self._data_changed)
        grid.addWidget(self.ovl, 0, 1)
        grid.addWidget(QtGui.QLabel("ROI"), 1, 0)
        self.roi = RoiCombo(self.ivm)
        grid.addWidget(self.roi, 1, 1)

        self.n_comp = NumericOption("Number of components", grid, 2, minval=1, maxval=3, default=3, intonly=True)
        self.compactness = NumericOption("Compactness", grid, 3, minval=0.01, maxval=1, step=0.05, default=0.1, intonly=False)
        self.sigma = NumericOption("Smoothing", grid, 4, minval=0, maxval=5, step=0.1, default=1, intonly=False)
        self.n_supervoxels = NumericOption("Number of supervoxels", grid, 5, minval=2, maxval=1000, default=20, intonly=True)

        grid.addWidget(QtGui.QLabel("Output name"), 6, 0)
        self.output_name = QtGui.QLineEdit("supervoxels")
        grid.addWidget(self.output_name, 6, 1)

        self.gen_btn = QtGui.QPushButton('Generate', self)
        self.gen_btn.clicked.connect(self._generate)
        grid.addWidget(self.gen_btn, 7, 0)
        hbox.addWidget(optbox)
        hbox.addStretch(1)
        layout.addLayout(hbox)

        layout.addStretch(1)

    def _data_changed(self, _):
        name = self.ovl.currentText()
        if name:
            ovl = self.ivm.data[name]
            self.n_comp.label.setVisible(ovl.nvols > 1)
            self.n_comp.spin.setVisible(ovl.nvols > 1)

    def batch_options(self):
        options = {
            "data" : self.ovl.currentText(),
            "roi" : self.roi.currentText(),
            "n-components" : self.n_comp.spin.value(),
            "compactness" : self.compactness.spin.value(),
            "sigma" : self.sigma.spin.value(),
            "n-supervoxels" :  self.n_supervoxels.spin.value(),
            "output-name" :  self.output_name.text() 
        }
        return "Supervoxels", options

    def _generate(self):
        process = SupervoxelsProcess(self.ivm, sync=True)
        process.run(self.batch_options()[1])



"""
Quantiphyse - PerfSLIC implementation

Copyright (c) 2013-2018 University of Oxford
"""
from .slic_superpixels_feat import slic_feat
from .perfusionslice_class import PerfSLIC

__all__ = ['slic_feat', 'PerfSLIC']



"""
Quantiphyse - Analysis process for supervoxel clustering

Copyright (c) 2013-2018 University of Oxford
"""

import numpy as np

from quantiphyse.processes import Process
from quantiphyse.processes.feat_pca import PcaFeatReduce
from quantiphyse.data import NumpyData

from .perfusionslic import slic_feat

class SupervoxelsProcess(Process):
    """
    Process to run 3d supervoxel generation
    """
    PROCESS_NAME = "Supervoxels"

    def __init__(self, ivm, **kwargs):
        Process.__init__(self, ivm, **kwargs)

    def preprocess_pca(self, img, ncomp, normalise_input_image=True, norm_type='perc'):
        """
        Do PCA analysis on 4D image to turn it into a 3D image suitable for SLIC
        """
        # Normalize enhancement curves using first 3 points (FIXME magic number)
        baseline = np.mean(img[:, :, :, :3], axis=-1)
        img = img - np.tile(np.expand_dims(baseline, axis=-1), (1, 1, 1, img.shape[-1]))
        
        # Run PCA feature extraction
        fe = PcaFeatReduce(n_components=ncomp, norm_modes=True, norm_input=normalise_input_image, norm_type=norm_type)
        feat_image = fe.get_training_features(img, smooth_timeseries=2.0, feature_volume=True)
        return feat_image

    def run(self, options):
        data = self.get_data(options)
        roi = self.get_roi(options, data.grid)
        comp = options.pop('compactness', 0.1)
        n_supervoxels = options.pop('n-supervoxels')
        sigma = options.pop('sigma', 1)
        recompute_seeds = options.pop('recompute-seeds', True)
        seed_type = options.get('seed-type', 'nrandom')
        output_name = options.pop('output-name', "supervoxels")

        img = data.raw()
        slices = roi.get_bounding_box()
        img = img[slices]
        mask = roi.raw()[slices]

        if img.ndim > 3 and img.shape[3] > 1:
            # For 4D data, use PCA to reduce down to 3D
            ncomp = options.pop('n-components', 3)
            img = self.preprocess_pca(img, ncomp)
        else:
            # For 3D data scale to a range of 0-1
            if img.ndim > 3: img = np.squeeze(img, -1)
            img = img.astype(np.float32)
            img = (img - img.min()) / (img.max() - img.min())

        # FIXME enforce_connectivity=True does not seem to work in ROI mode?
        labels = slic_feat(img, n_segments=n_supervoxels, compactness=comp, sigma=sigma,
                           seed_type=seed_type, multichannel=False, multifeat=True,
                           enforce_connectivity=False, return_adjacency=False, spacing=data.grid.spacing,
                           mask=mask, recompute_seeds=recompute_seeds, n_random_seeds=n_supervoxels)
        newroi = np.zeros(data.grid.shape)
        newroi[slices] = np.array(labels, dtype=np.int) + 1
        self.ivm.add(newroi, grid=data.grid, name=output_name, roi=True, make_current=True)



"""
Quantiphyse - SLIC implementation for supervoxels

Copyright (c) 2013-2018 University of Oxford
"""

# coding=utf-8

from __future__ import division, print_function, absolute_import

import collections as coll
import warnings

import numpy as np
from scipy import ndimage as ndi
from scipy.ndimage.morphology import distance_transform_edt
from skimage.util import img_as_float, regular_grid
from skimage.color import rgb2lab

from ._slic_feat import _slic_feat_cython, _enforce_label_connectivity_cython, _find_adjacency_map, _find_adjacency_map_mask

from .additional.processing import get_mpd

def slic_feat(image, n_segments=100, compactness=10., max_iter=10, sigma=0,
              seed_type='grid', spacing=None, multichannel=True, convert2lab=None,
              enforce_connectivity=False, min_size_factor=0.5, max_size_factor=3,
              slic_zero=False, multifeat=True, return_adjacency=False, mask=None,
              recompute_seeds=False, n_random_seeds=10):

    """Segments image using k-means clustering in Color-(x,y,z) space.

    Parameters
    ----------
    image : 2D, 3D or 4D ndarray
        Input image, which can be 2D or 3D, and grayscale or multichannel
        (see `multichannel` parameter).
    n_segments : int, optional
        The (approximate) number of labels in the segmented output image.
    compactness : float, optional
        Balances color-space proximity and image-space proximity. Higher
        values give more weight to image-space. As `compactness` tends to
        infinity, superpixel shapes become square/cubic. In SLICO mode, this
        is the initial compactness.
    max_iter : int, optional
        Maximum number of iterations of k-means.
    sigma : float or (3,) array-like of floats, optional
        Width of Gaussian smoothing kernel for pre-processing for each
        dimension of the image. The same sigma is applied to each dimension in
        case of a scalar value. Zero means no smoothing.
        Note, that `sigma` is automatically scaled if it is scalar and a
        manual voxel spacing is provided (see Notes section).
    spacing : (3,) array-like of floats, optional
        The voxel spacing along each image dimension. By default, `slic`
        assumes uniform spacing (same voxel resolution along z, y and x).
        This parameter controls the weights of the distances along z, y,
        and x during k-means clustering.
    multichannel : bool, optional
        Whether the last axis of the image is to be interpreted as multiple
        channels or another spatial dimension.
    convert2lab : bool, optional
        Whether the input should be converted to Lab colorspace prior to
        segmentation. The input image *must* be RGB. Highly recommended.
        This option defaults to ``True`` when ``multichannel=True`` *and*
        ``image.shape[-1] == 3``.
    enforce_connectivity: bool, optional (default False)
        Whether the generated segments are connected or not
    min_size_factor: float, optional
        Proportion of the minimum segment size to be removed with respect
        to the supposed segment size ```depth*width*height/n_segments```
    max_size_factor: float, optional
        Proportion of the maximum connected segment size. A value of 3 works
        in most of the cases.
    slic_zero: bool, optional
        Run SLIC-zero, the zero-parameter mode of SLIC. [2]_
    mask: ndarray of bools or 0s and 1s, optional
        Array of same shape as `image`. Supervoxel analysis will only be performed on points at
        which mask == True

    Returns
    -------
    labels : 2D or 3D array
        Integer mask indicating segment labels.

    Raises
    ------
    ValueError
        If ``convert2lab`` is set to ``True`` but the last array
        dimension is not of length 3.

    Notes
    -----
    * If `sigma > 0`, the image is smoothed using a Gaussian kernel prior to
      segmentation.

    * If `sigma` is scalar and `spacing` is provided, the kernel width is
      divided along each dimension by the spacing. For example, if ``sigma=1``
      and ``spacing=[5, 1, 1]``, the effective `sigma` is ``[0.2, 1, 1]``. This
      ensures sensible smoothing for anisotropic images.

    * The image is rescaled to be in [0, 1] prior to processing.

    * Images of shape (M, N, 3) are interpreted as 2D RGB images by default. To
      interpret them as 3D with the last dimension having length 3, use
      `multichannel=False`.

    References
    ----------
    .. [1] Radhakrishna Achanta, Appu Shaji, Kevin Smith, Aurelien Lucchi,
        Pascal Fua, and Sabine Susstrunk, SLIC Superpixels Compared to
        State-of-the-art Superpixel Methods, TPAMI, May 2012.
    .. [2] http://ivrg.epfl.ch/research/superpixels#SLICO

    Examples
    --------
    >>> from skimage.segmentation import slic
    >>> from skimage.data import astronaut
    >>> img = astronaut()
    >>> segments = slic(img, n_segments=100, compactness=10)

    Increasing the compactness parameter yields more square regions:

    >>> segments = slic(img, n_segments=100, compactness=20)

    """
    if enforce_connectivity is None:
        warnings.warn('Deprecation: enforce_connectivity will default to'
                      ' True in future versions.')
        enforce_connectivity = False

    if mask is None and seed_type == 'nrandom':
        warnings.warn('nrandom assignment of seed points should only be used with an ROI. Changing seed type.')
        seed_type = 'grid'

    if seed_type == 'nrandom' and recompute_seeds is False:
        warnings.warn('Seeds should be recomputed when seed points are randomly assigned')

    image = img_as_float(image)
    is_2d = False
    if image.ndim == 2:
        # 2D grayscale image
        image = image[np.newaxis, ..., np.newaxis]
        is_2d = True
    elif image.ndim == 3 and multichannel:
        # Make 2D multichannel image 3D with depth = 1
        image = image[np.newaxis, ...]
        is_2d = True
    elif image.ndim == 3 and not multichannel:
        # Add channel as single last dimension
        image = image[..., np.newaxis]

    if spacing is None:
        spacing = np.ones(3)
    elif isinstance(spacing, (list, tuple)):
        spacing = np.array(spacing, dtype=np.double)

    if not isinstance(sigma, coll.Iterable):
        sigma = np.array([sigma, sigma, sigma], dtype=np.double)
        sigma /= spacing.astype(np.double)
    elif isinstance(sigma, (list, tuple)):
        sigma = np.array(sigma, dtype=np.double)
    if (sigma > 0).any():
        # add zero smoothing for multichannel dimension
        sigma = list(sigma) + [0]
        image = ndi.gaussian_filter(image, sigma)

    if multichannel and (convert2lab or convert2lab is None):
        if image.shape[-1] != 3 and convert2lab:
            raise ValueError("Lab colorspace conversion requires a RGB image.")
        elif image.shape[-1] == 3:
            image = rgb2lab(image)

    if multifeat is True:
        feat_scale = float(image.shape[3])
    else:
        feat_scale = 1.0

    depth, height, width = image.shape[:3]

    if mask is None:
        mask = np.ones(image.shape[:3], dtype=np.bool)
    else:
        mask = np.asarray(mask, dtype=np.bool)

    if seed_type == 'nrandom':

        segments_z = np.zeros(n_random_seeds, dtype=int)
        segments_y = np.zeros(n_random_seeds, dtype=int)
        segments_x = np.zeros(n_random_seeds, dtype=int)

        m_inv = np.copy(mask)

        # SEED STEP 1:  n seeds are placed as far as possible from every other seed and the edge.
        for ii in range(n_random_seeds):

            dtrans = distance_transform_edt(m_inv, sampling=spacing)

            mcoords = np.nonzero(dtrans == np.max(dtrans))

            segments_z[ii] = int(mcoords[2][0])
            segments_y[ii] = int(mcoords[1][0])
            segments_x[ii] = int(mcoords[0][0])

            m_inv[segments_x[ii], segments_y[ii], segments_z[ii]] = False

            # plt.imshow(dtrans[:, :, segments_z[ii]])
            # plt.show()

        segments_color = np.zeros((segments_z.shape[0], image.shape[3]))
        segments = np.concatenate([segments_x[..., np.newaxis],
                                   segments_y[..., np.newaxis],
                                   segments_z[..., np.newaxis],
                                   segments_color], axis=1)

        sx = np.ascontiguousarray(segments_x, dtype=np.int32)
        sy = np.ascontiguousarray(segments_y, dtype=np.int32)
        sz = np.ascontiguousarray(segments_z, dtype=np.int32)

        out1 = get_mpd(sx, sy, sz)
        step_x, step_y, step_z = out1[0], out1[1], out1[2]

    elif seed_type == 'grid':

        # initialize cluster centroids for desired number of segments
        # essentially just outputs the indices of a grid in the x, y and z direction
        grid_z, grid_y, grid_x = np.mgrid[:depth, :height, :width]
        # returns 3 slices (an object representing an array of slices, see builtin slice)
        slices = regular_grid(image.shape[:3], n_segments)
        step_z, step_y, step_x = [int(s.step) for s in slices]  # extract step size from slices
        segments_z = grid_z[slices]  # use slices to extract coordinates for centre points
        segments_y = grid_y[slices]
        segments_x = grid_x[slices]

        # mask_ind = mask[slices].reshape(-1)
        # list of all locations as well as zeros for the color features
        segments_color = np.zeros(segments_z.shape + (image.shape[3],))
        segments = np.concatenate([segments_z[..., np.newaxis],
                                   segments_y[..., np.newaxis],
                                   segments_x[..., np.newaxis],
                                   segments_color],
                                  axis=-1).reshape(-1, 3 + image.shape[3])
    else:
        raise ValueError('seed_type should be nrandom or grid')

    # Only use values in the mask
    # segments = segments[mask_ind, :]

    #print("Number of supervoxels: ", segments.shape[0])
    segments = np.ascontiguousarray(segments)

    # we do the scaling of ratio in the same way as in the SLIC paper
    # so the values have the same meaning
    step = float(max((step_z, step_y, step_x)))
    ratio = 1.0 / compactness

    image = np.ascontiguousarray(image * ratio, dtype=np.double)
    mask = np.ascontiguousarray(mask, dtype=np.int32)

    if recompute_seeds:

        # Seed step 2: Run SLIC to reinitialise seeds
        # Runs the supervoxel method but only uses distance to better initialise the method
        labels = _slic_feat_cython(image, mask, segments, step, max_iter, spacing, slic_zero, feat_scale,
                                   only_dist=True)

        # # Testing
        # fig = plt.figure()
        # ax = fig.add_subplot(111, projection='3d')
        # ax.scatter(segments_old[:, 0], segments_old[:, 1], segments_old[:, 2], c='red', s=80)
        # ax.scatter(segments[:, 0], segments[:, 1], segments[:, 2], c='blue', s=80)
        # plt.show()

    labels = _slic_feat_cython(image, mask, segments, step, max_iter, spacing, slic_zero, feat_scale,
                               only_dist=False)

    if enforce_connectivity:
        segment_size = depth * height * width / n_segments
        min_size = int(min_size_factor * segment_size)
        max_size = int(max_size_factor * segment_size)
        labels = _enforce_label_connectivity_cython(labels,
                                                    mask,
                                                    n_segments,
                                                    min_size,
                                                    max_size)

    # Also return adjacency map
    if return_adjacency:
        labels = np.ascontiguousarray(labels, dtype=np.int32)
        if mask is None:
            adj_mat, border_mat = _find_adjacency_map(labels)
        else:
            adj_mat, border_mat = _find_adjacency_map_mask(labels)

        #print(adj_mat.shape)
        if is_2d:
            labels = labels[0]
        return labels, adj_mat, border_mat

    else:

        if is_2d:
            labels = labels[0]
        return labels



from __future__ import print_function, division, absolute_import

import nibabel as nib
import numpy as np


def save_nifti(data, folder1, name1, hdr=None):
    """
    Save as a nifti file
    @param data: Image data
    @param folder1: Folder to save
    @param name1: Saving name
    @param example_nifti: Provide a path to an example Nifti in order to save with correct affine matrix and other info
    @return:
    """

    # 1) Generic header information
    if hdr is None:
        # Default image creation
        img = nib.Nifti1Image(data, np.eye(4))

    # Header information derived from a specific nifti header
    else:
        img = nib.Nifti1Image(data, hdr.get_best_affine(), header=hdr)
        # Harmonise header with data
        img.update_header()

    # 2) check if it has a backslash or not
    if folder1[-1] == '/':
        img.to_filename(folder1 + name1)
    else:
        img.to_filename(folder1 + '/' + name1)




"""
Library to speed up B-spline processing of the whole image
"""

from __future__ import division, print_function

import numpy as np
cimport numpy as np
from scipy.interpolate import LSQUnivariateSpline, splrep, splev


def image_bspline_smooth(double[:, :, :, ::1] im4d, double[::1] splpos):

    """
    Apply B-spline smoothing to the whole image
    @param im4d: 4D image
    @param splpos: knot locations - positions in the time domain
    @return:
    """

    cdef Py_ssize_t nx =im4d.shape[0]
    cdef Py_ssize_t ny =im4d.shape[1]
    cdef Py_ssize_t nz =im4d.shape[2]
    cdef Py_ssize_t nt =im4d.shape[3]

    cdef double[::1] vec1 = np.zeros(nt, dtype=np.double)
    cdef double[::1] vec3 = np.zeros(nt, dtype=np.double)
    cdef double[::1] r1 = np.zeros(nt, dtype=np.double)

    cdef Py_ssize_t xx, yy, zz, ii

    for ii in range(nt):
        r1[ii] = ii

    for xx in range(ny):
        print(xx)
        for yy in range(ny):
            for zz in range(nz):

                # populate vector
                for ii in range(nt):
                    r1[ii] = ii
                    vec1[ii] = im4d[xx, yy, zz, ii]

                s = LSQUnivariateSpline(r1, vec1, splpos, k=4)
                #print(s.get_knots())
                vec3 = s(r1)

                for ii in range(nt):
                    im4d[xx, yy, zz, ii] = vec3[ii]

    return np.asarray(im4d)









"""
Quantiphyse - Tests for supervoxel clustering widget

Copyright (c) 2013-2018 University of Oxford
"""
import unittest 

import numpy as np

from quantiphyse.processes import Process
from quantiphyse.test import WidgetTest, ProcessTest

from .widgets import PerfSlicWidget

NUM_SV = 4
NAME = "test_sv"
NUM_PCA = 3
SIGMA = 0.5
COMP = 0.2

class PerfSlicWidgetTest(WidgetTest):

    def widget_class(self):
        return PerfSlicWidget

    def testNoData(self):
        """ User clicks the generate buttons with no data"""
        self.harmless_click(self.w.gen_btn)

    def test3dData(self):
        self.ivm.add(self.data_3d, grid=self.grid, name="data_3d")
        self.w.ovl.setCurrentIndex(0)
        self.processEvents()
        self.assertFalse(self.w.n_comp.spin.isVisible())

        self.w.compactness.spin.setValue(COMP)
        self.w.sigma.spin.setValue(SIGMA)
        self.w.n_supervoxels.spin.setValue(NUM_SV)
        self.w.output_name.setText(NAME)

        self.harmless_click(self.w.gen_btn)
        self.processEvents()

        self.assertTrue(NAME in self.ivm.rois)
        self.assertEquals(self.ivm.current_roi.name, NAME)
        # Don't check regions as we know this doesn't work. Really you shouldn't be running
        # supervoxels without an ROI, we are just checking it handles it gracefully if you do
        #self.assertEquals(len(self.ivm.rois[NAME].regions, NUM_SV)
        self.assertFalse(self.error)

    def test3dDataMask(self):
        self.ivm.add(self.mask, grid=self.grid, name="mask", roi=True)
        self.w.ovl.setCurrentIndex(0)
        self.test3dData()
        self.assertEquals(len(self.ivm.rois[NAME].regions), NUM_SV)
        # Supervoxel value is always zero outside the ROI
        sv = self.ivm.rois[NAME].raw()
        self.assertTrue(np.all(sv[self.mask == 0] == 0))
        self.assertFalse(self.error)

    def test4dData(self):
        self.ivm.add(self.data_4d, grid=self.grid, name="data_4d")
        self.w.ovl.setCurrentIndex(0)
        self.w.roi.setCurrentIndex(0)
        self.processEvents()
        self.assertTrue(self.w.n_comp.spin.isEnabled())

        self.w.compactness.spin.setValue(COMP)
        self.w.n_comp.spin.setValue(NUM_PCA)
        self.w.sigma.spin.setValue(SIGMA)
        self.w.n_supervoxels.spin.setValue(NUM_SV)
        self.w.output_name.setText(NAME)

        self.harmless_click(self.w.gen_btn)
        self.processEvents()

        self.assertTrue(NAME in self.ivm.rois)
        self.assertEquals(self.ivm.current_roi.name, NAME)
        # Don't check regions as we know this doesn't work. Really you shouldn't be running
        # supervoxels without an ROI, we are just checking it handles it gracefully if you do
        #self.assertEquals(len(self.ivm.rois[NAME].regions, NUM_SV)
        self.assertFalse(self.error)

    def test4dDataMask(self):
        self.ivm.add(self.mask, grid=self.grid, name="mask", roi=True)
        self.w.ovl.setCurrentIndex(0)
        self.w.roi.setCurrentIndex(0)
        self.test4dData()
        self.assertEquals(len(self.ivm.rois[NAME].regions), NUM_SV)
        # Supervoxel value is always zero outside the ROI
        sv = self.ivm.rois[NAME].raw()
        self.assertTrue(np.all(sv[self.mask == 0] == 0))
        self.assertFalse(self.error)

class SupervoxelsProcessTest(ProcessTest):

    def test3d(self):
        yaml = """
  - Supervoxels:
      data: data_3d
      output-name: sv_3d
      compactness: 0.01
      n-supervoxels: 20
"""
        self.run_yaml(yaml)
        self.assertEqual(self.status, Process.SUCCEEDED)
        self.assertTrue("sv_3d" in self.ivm.rois)

    def test4d(self):
        yaml = """

  - Supervoxels:
      data: data_4d
      output-name: sv_4d
      n-components: 3
      compactness: 0.02
      n-supervoxels: 30
"""
        self.run_yaml(yaml)
        self.assertEqual(self.status, Process.SUCCEEDED)
        self.assertTrue("sv_4d" in self.ivm.rois)

if __name__ == '__main__':
    unittest.main()



"""
Processing labels in an efficient way using Cython
"""

from libc.float cimport DBL_MAX
import numpy as np
cimport numpy as cnp

def createlab(Py_ssize_t[:, :, ::1] slic_reg, double[::1] label_prob):
    """
    Create a labelled image based on the labels assigned to each region
    @param slic_reg: Regions to relabel
    @param label_prob: New label to assign. The indices of this label vector correspond to the
    supervoxel label in slic_reg
    @return:
    """

    cdef double[:, :, ::1] label_img_prob = np.zeros_like(slic_reg, dtype=np.double)
    cdef Py_ssize_t[:, :, ::1] subset = np.zeros_like(slic_reg, dtype=np.intp)

    a1 = np.shape(slic_reg)

    #Loop over each voxel and assign new label
    for xx in range(a1[0]):
        for yy in range(a1[1]):
            for zz in range(a1[2]):
                label_img_prob[xx, yy, zz] = label_prob[slic_reg[xx, yy, zz]]

    # Output new labelled array
    return np.asarray(label_img_prob)


def grouplab(Py_ssize_t[:, :, ::1] slic_reg, double[::1] label_prob, double[:,:,:,::1] img_feat):
    """
    Extract region matching a list of labels
    @param slic_reg: Regions to relabel
    @param label_prob: New label to assign
    @return:
    """

    a1 = np.shape(slic_reg)

    #Loop over each voxel and assign new label
    cdef int dd = 0
    for xx in range(a1[0]):
        for yy in range(a1[1]):
            for zz in range(a1[2]):
                if slic_reg[xx, yy, zz] in label_prob:
                    dd += 1

    sf1 = np.shape(img_feat)[-1]

    cdef Py_ssize_t[:, ::1] reg_inert = np.zeros((dd, 3), dtype=np.intp)
    cdef double[:, ::1] feat_inert = np.zeros((dd, sf1), dtype=np.double)

    #Loop over each voxel and assign new label
    cdef int cc = 0
    for xx in range(a1[0]):
        for yy in range(a1[1]):
            for zz in range(a1[2]):

                if slic_reg[xx, yy, zz] in label_prob:
                    reg_inert[cc, 0] = xx
                    reg_inert[cc, 1] = yy
                    reg_inert[cc, 2] = zz

                    for ii in range(sf1):
                        feat_inert[cc, ii] = img_feat[xx, yy, zz, ii]

                    cc += 1

    # Output new labelled array
    return np.asarray(reg_inert), np.asarray(feat_inert)




"""
Quantiphyse - supervoxel clustering package

Copyright (c) 2013-2018 University of Oxford
"""

from .widgets import PerfSlicWidget
from .process import SupervoxelsProcess
from .tests import PerfSlicWidgetTest, SupervoxelsProcessTest

QP_MANIFEST = {
    "widgets" : [PerfSlicWidget],
    "widget-tests" : [PerfSlicWidgetTest,],
    "process-tests" : [SupervoxelsProcessTest,],
    "processes" : [SupervoxelsProcess],
}
